#include "stringutils.h"
extern "C"
{
#include "compress.h"
}

#include <iostream>
#include <vector>
#include <filesystem>
#include <cstring>
#include <cstdio>
#include <cerrno>
extern "C"
{
#include <lzma.h>
}

const uint32_t COMPRESSION_PRESET = 5;
const std::vector<std::string> ENDINGS = {".mf4", ".rif"};

int main(int argc, char *argv[])
{
    const std::filesystem::path directory = argc > 1 ? argv[1] : std::filesystem::current_path();
    lzma_stream strm = LZMA_STREAM_INIT;
    bool success = true;

    for (const auto &entry : std::filesystem::directory_iterator(directory))
    {
        const std::string path = entry.path().string();

        if (!entry.is_regular_file() || !stringutils::has_any_ending(stringutils::to_lower(path), ENDINGS))
        {
            continue;
        }

        std::cout << "Compressing " + path + "..." << std::flush;

        if (!lzma_init_encoder(&strm, COMPRESSION_PRESET))
        {
            success = false;
            break;
        }

        const std::string output_path = path + ".xz";

        FILE *infile = std::fopen(path.c_str(), "rb");
        FILE *outfile = std::fopen(output_path.c_str(), "wb");

        success = lzma_compress(&strm, infile, outfile);

        std::fclose(infile);

        if (std::fclose(outfile))
        {
            fprintf(stderr, "Write error: %s\n", std::strerror(errno));
            success = false;
        }

        if (!success) {
            std::cout << " FAILED!" << std::endl;
            break;
        }

        std::filesystem::remove(path);

        std::cout << " OK!" << std::endl;
    }

    lzma_end(&strm);

    return success ? EXIT_SUCCESS : EXIT_FAILURE;
}
